@@ -15,12 +15,12 @@ class Elpa(package.Package):
   def __init__(self,argdb,log):
     package.Package.__init__(self,argdb,log)
     self.packagename    = 'elpa'
+    self.packagetype    = 'gnu'
     self.installable    = True
     self.downloadable   = True
-    self.version        = '2021.11.002'
+    self.version        = '2022.11.001'
     self.archive        = 'elpa-'+self.version+'.tar.gz'
     self.url            = 'https://elpa.mpcdf.mpg.de/software/tarball-archive/Releases/'+self.version+'/'+self.archive
-    self.dirname        = 'elpa-'+self.version
     self.supportssingle = True
     self.fortran        = True
     self.ProcessArgs(argdb)
@@ -53,43 +53,45 @@ class Elpa(package.Package):
   def Check(self,slepcconf,slepcvars,petsc,archdir):
     code = self.SampleCode(petsc)
     if self.packagedir:
-      dirs = [os.path.join(self.packagedir,'lib'),self.packagedir,os.path.join(self.packagedir,'lib64')]
-      incdirs = [os.path.join(self.packagedir,'include'),self.packagedir]
-    else:
-      dirs = self.GenerateGuesses('elpa',archdir)
-      incdirs = self.GenerateGuesses('elpa',archdir,'include')
-
-    libs = self.packagelibs
-    if not libs:
-      libs = ['-lelpa']
-    includes = self.packageincludes
-    if not includes:
-      includes = ['.']
-
-    for (d,i) in zip(dirs,incdirs):
-      if d:
-        if petsc.buildsharedlib:
-          l = [petsc.slflag + d] + ['-L' + d] + libs
-        else:
-          l = ['-L' + d] + libs
-        f = ['-I' + i]
+      if os.path.isdir(os.path.join(os.sep,'usr','lib64')):
+        dirs = ['',os.path.join(self.packagedir,'lib64'),self.packagedir,os.path.join(self.packagedir,'lib')]
       else:
-        l = libs
-        f = ['-I' + includes[0]]
-      result = self.Link([],[],l+f,code,' '.join(f),petsc.language)
-      if result:
-        slepcconf.write('#define SLEPC_HAVE_ELPA 1\n')
-        slepcvars.write('ELPA_LIB = ' + ' '.join(l) + '\n')
-        slepcvars.write('ELPA_INCLUDE = ' + ' '.join(f) + '\n')
-        self.havepackage = True
-        self.packageflags = l+f
-        return
+        dirs = ['',os.path.join(self.packagedir,'lib'),self.packagedir,os.path.join(self.packagedir,'lib64')]
+      incdirs = ['',os.path.join(self.packagedir,'include'),self.packagedir]
+    else:
+      dirs = self.GenerateGuesses('elpa',archdir) + self.GenerateGuesses('elpa',archdir,'lib64')
+      incdirs = self.GenerateGuesses('elpa',archdir,'include') + self.GenerateGuesses('elpa',archdir,os.path.join('include','elpa-'+self.version))
+
+    libs = [self.packagelibs] if self.packagelibs else ['-lelpa']
+    includes = [self.packageincludes] if self.packageincludes else []
+
+    for d in dirs:
+      for i in incdirs:
+        if d:
+          if petsc.buildsharedlib:
+            l = [self.slflag + d] + ['-L' + d] + libs
+          else:
+            l = ['-L' + d] + libs
+          f = (['-I' + i] if i else [])
+        else:
+          l = libs
+          f = []
+        (result, output) = self.Link([],[],' '.join(l+f),code,' '.join(f),petsc.language)
+        if result:
+          self.libflags = ' '.join(l)
+          self.includeflags = ' '.join(f)
+          slepcconf.write('#define SLEPC_HAVE_ELPA 1\n')
+          slepcvars.write('ELPA_LIB = ' + self.libflags + '\n')
+          slepcvars.write('ELPA_INCLUDE = ' + self.includeflags + '\n')
+          self.havepackage = True
+          self.packageflags = ' '.join(l+f)
+          return
 
     self.log.Exit('Unable to link with ELPA library in directories'+' '.join(dirs)+' with libraries and link flags '+' '.join(libs))
 
 
   def DownloadAndInstall(self,slepcconf,slepcvars,slepc,petsc,archdir,prefixdir):
-    externdir = slepc.CreateDir(archdir,'externalpackages')
+    externdir = slepc.GetExternalPackagesDir(archdir)
     builddir  = self.Download(externdir,slepc.downloaddir)
     incdir,libdir = slepc.CreatePrefixDirs(prefixdir)
 
@@ -99,12 +101,12 @@ class Elpa(package.Package):
       self.log.Exit('--download-elpa requires that the command autoreconf is available on your PATH')
 
     # Build package
-    confopt = '--prefix='+prefixdir+' CC="'+petsc.cc+'" CFLAGS="'+petsc.getCFlags()+'" F77="'+petsc.fc+'" FFLAGS="'+petsc.getFFlags()+'" FC="'+petsc.fc+'" FCFLAGS="'+petsc.getFFlags()+'" LIBS="'+petsc.blaslapack_lib+'" SCALAPACK_LDFLAGS="'+petsc.scalapack_lib+'" --disable-sse --disable-sse-assembly --disable-avx --disable-avx2 --disable-avx512'
+    confopt = ['--prefix='+prefixdir, '--libdir='+os.path.join(prefixdir,'lib'), 'CC="'+petsc.cc+'"', 'CFLAGS="'+petsc.getCFlags()+'"', 'F77="'+petsc.fc+'"', 'FFLAGS="'+petsc.getFFlags()+'"', 'FC="'+petsc.fc+'"', 'FCFLAGS="'+petsc.getFFlags()+'"', 'CXX="'+petsc.cxx+'"', 'CXXFLAGS="'+petsc.getCXXFlags()+'"', 'CPP="'+petsc.cpp+'"', 'LIBS="'+petsc.blaslapack_lib+'"', 'SCALAPACK_LDFLAGS="'+petsc.scalapack_lib+'"', '--disable-sse', '--disable-sse-assembly', '--disable-avx', '--disable-avx2', '--disable-avx512']
     if petsc.mpiuni or petsc.msmpi:
-      confopt = confopt+' --with-mpi=no'
+      confopt = confopt + ['--with-mpi=no']
     if petsc.precision == 'single':
-      confopt = confopt+' --enable-single-precision'
-    (result,output) = self.RunCommand('cd '+builddir+'&& ./configure '+confopt+' && '+petsc.make+' -j'+petsc.make_np+' && '+petsc.make+' install')
+      confopt = confopt + ['--enable-single-precision']
+    (result,output) = self.RunCommand('cd '+builddir+'&& ./configure '+' '.join(confopt)+' '+self.buildflags+' && '+petsc.make+' -j'+petsc.make_np+' && '+petsc.make+' install')
     if result:
       self.log.Exit('Installation of ELPA failed')
 
@@ -113,21 +115,23 @@ class Elpa(package.Package):
     altlibdir = os.path.join(prefixdir,'lib64')
     for ldir in [libdir,altlibdir]:
       if petsc.buildsharedlib:
-        l = petsc.slflag + ldir + ' -L' + ldir + ' -lelpa'
+        l = self.slflag + ldir + ' -L' + ldir + ' -lelpa'
       else:
         l = '-L' + ldir + ' -lelpa'
-      f = '-I' + os.path.join(incdir,self.dirname)
-      (result, output) = self.Link([],[],[l]+[f],code,f,petsc.language)
+      f = '-I' + os.path.join(incdir,self.GetDirectoryName())
+      (result, output) = self.Link([],[],l+' '+f,code,f,petsc.language)
       if result: break
 
     if not result:
       self.log.Exit('Unable to link with downloaded ELPA')
 
     # Write configuration files
+    self.libflags = l
+    self.includeflags = f
     slepcconf.write('#define SLEPC_HAVE_ELPA 1\n')
-    slepcvars.write('ELPA_LIB = ' + l + '\n')
-    slepcvars.write('ELPA_INCLUDE = ' + f + '\n')
+    slepcvars.write('ELPA_LIB = ' + self.libflags + '\n')
+    slepcvars.write('ELPA_INCLUDE = ' + self.includeflags + '\n')
 
     self.havepackage = True
-    self.packageflags = [l] + [f]
+    self.packageflags = l+' '+f
 
