@@ -1,7 +1,7 @@
 #
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #  SLEPc - Scalable Library for Eigenvalue Problem Computations
-#  Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
+#  Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 #
 #  This file is part of SLEPc.
 #  SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -16,24 +16,48 @@ class PETSc(package.Package):
     package.Package.__init__(self,argdb,log)
     self.packagename = 'petsc'
 
+  def __str__(self):
+    ''' String conversion, used when writing the configure-hash file '''
+    if not hasattr(self,'dir'): return ''
+    conf = 'PETSC_DIR=' + self.dir + '\n'
+    if not hasattr(self,'lversion'): return conf
+    conf += 'PETSc version: ' + self.lversion + '\n'
+    if not hasattr(self,'arch'): return conf
+    conf += 'PETSC_ARCH=' + self.arch + '\n'
+    conf += 'PETSC_SCALAR=' + self.scalar + '\n'
+    conf += 'PETSC_PRECISION=' + self.precision + '\n'
+    conf += 'BLASLAPACK_LIB=' + self.blaslapack_lib + '\n'
+    conf += 'CC=' + self.cc + '\n'
+    conf += 'CC_FLAGS=' + self.cc_flags + '\n'
+    if hasattr(self,'fc'):
+      conf += 'FC=' + self.fc + '\n'
+      conf += 'FC_FLAGS=' + (self.fc_flags if hasattr(self,'fc_flags') else '') + '\n'
+    if hasattr(self,'cxx'):
+      conf += 'CXX=' + self.cxx + '\n'
+      conf += 'CXX_FLAGS=' + (self.cxx_flags if hasattr(self,'cxx_flags') else '') + '\n'
+    conf += 'PETSc configure options:\n'
+    args = sorted(set(self.configure_options.split()))
+    conf += '\n'.join('    '+a for a in args) + '\n'
+    return conf
+
   def ShowHelp(self):
     pass
 
   def Check(self):
-    (result, output) = self.Link([],[],[])
+    (result, output) = self.Link([],[],'')
     self.havepackage = result
 
   def InitDir(self,prefixdir):
     if 'PETSC_DIR' in os.environ:
       self.dir = os.path.normpath(os.environ['PETSC_DIR'])
       if not os.path.exists(self.dir):
-        self.log.Exit('PETSC_DIR enviroment variable is not valid')
+        self.log.Exit('PETSC_DIR environment variable is not valid')
     else:
       if prefixdir:
         self.dir = prefixdir
         os.environ['PETSC_DIR'] = self.dir
       else:
-        self.log.Exit('PETSC_DIR enviroment variable is not set')
+        self.log.Exit('PETSC_DIR environment variable is not set')
 
   def LoadVersion(self):
     try:
@@ -75,66 +99,36 @@ class PETSc(package.Package):
         suggest = os.path.basename(self.arch)
         if not suggest: suggest = os.path.basename(self.arch[0:-1])
         self.log.Exit('Variable PETSC_ARCH must not be a full path\nYou set PETSC_ARCH=%s, maybe you meant PETSC_ARCH=%s'% (self.arch,suggest))
-      petscvariables = os.path.join(self.dir,self.arch,'lib','petsc','conf','petscvariables')
+      self.petscvariables = os.path.join(self.dir,self.arch,'lib','petsc','conf','petscvariables')
       petscconf_h = os.path.join(self.dir,self.arch,'include','petscconf.h')
     else:
       self.isinstall = True
-      petscvariables = os.path.join(self.dir,'lib','petsc','conf','petscvariables')
+      self.petscvariables = os.path.join(self.dir,'lib','petsc','conf','petscvariables')
       petscconf_h = os.path.join(self.dir,'include','petscconf.h')
 
     self.buildsharedlib = False
     self.bfort = 'nobfortinpetsc'
     try:
-      f = open(petscvariables)
-      for l in f.readlines():
-        r = l.split('=',1)
-        if len(r)!=2: continue
-        k = r[0].strip()
-        v = r[1].strip()
-        v = v.replace('${PETSC_DIR}',self.dir)  # needed in some Cray installations
-        if k == 'PETSC_SCALAR':
-          self.scalar = v
-        elif k == 'PETSC_PRECISION':
-          self.precision = v
-        elif k == 'MAKE':
-          self.make = v
-        elif k == 'PREFIXDIR':
-          self.prefixdir = v
-        elif k == 'BFORT':
-          self.bfort = v
-        elif k == 'CC':
-          self.cc = v
-        elif k == 'CC_FLAGS':
-          self.cc_flags = v
-        elif k == 'CXX':
-          self.cxx = v
-        elif k == 'CXX_FLAGS':
-          self.cxx_flags = v
-        elif k == 'FC' and not v=='':
-          self.fc = v
-        elif k == 'FC_FLAGS':
-          self.fc_flags = v
-        elif k == 'AR':
-          self.ar = v
-        elif k == 'AR_FLAGS':
-          self.ar_flags = v
-        elif k == 'AR_LIB_SUFFIX':
-          self.ar_lib_suffix = v
-        elif k == 'BUILDSHAREDLIB' and v=='yes':
-          self.buildsharedlib = True
-        elif k == 'CC_LINKER_SLFLAG':
-          self.slflag = v
-        elif k == 'SL_LINKER_SUFFIX':
-          self.sl_suffix = v
-        elif k == 'RANLIB':
-          self.ranlib = v
-        elif k == 'BLASLAPACK_LIB':
-          self.blaslapack_lib = v
-        elif k == 'SCALAPACK_LIB':
-          self.scalapack_lib = v
-      f.close()
+      with open(self.petscvariables) as f:
+        for l in f.readlines():
+          r = l.split('=',1)
+          if len(r)!=2: continue
+          k = r[0].strip()
+          v = r[1].strip()
+          v = v.replace('${PETSC_DIR}',self.dir)  # needed in some Cray installations
+          if k == 'PETSC_SCALAR':
+            self.scalar = v
+          elif k == 'PETSC_PRECISION':
+            self.precision = v
+          elif k == 'FC' and not v=='':
+            self.fc = v
+          elif k == 'BUILDSHAREDLIB' and v=='yes':
+            self.buildsharedlib = True
+          else:
+            if k in ['AR','AR_FLAGS','AR_LIB_SUFFIX','BFORT','BLASLAPACK_LIB','CC','CC_FLAGS','CC_LINKER_SLFLAG','CMAKE','CONFIGURE_OPTIONS','CPP','CXX','CXX_FLAGS','FC_FLAGS','MAKE','MAKE_NP','PREFIXDIR','RANLIB','SCALAPACK_LIB','SEDINPLACE','SL_LINKER_SUFFIX']:
+              setattr(self,k.lower(),v)
     except:
-      self.log.Exit('Cannot process file ' + petscvariables)
+      self.log.Exit('Cannot process file ' + self.petscvariables)
 
     self.ind64 = False
     self.mpiuni = False
@@ -148,38 +142,41 @@ class PETSc(package.Package):
     self.cxxdialectcxx11 = False
     self.packages = []
     try:
-      f = open(petscconf_h)
-      for l in f.readlines():
-        l = l.split()
-        if len(l)==3 and l[0]=='#define' and l[1]=='PETSC_USE_64BIT_INDICES' and l[2]=='1':
-          self.ind64 = True
-        elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_HAVE_MPIUNI' and l[2]=='1':
-          self.mpiuni = True
-        elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_HAVE_MSMPI' and l[2]=='1':
-          self.msmpi = True
-        elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_USE_DEBUG' and l[2]=='1':
-          self.debug = True
-        elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_USE_SINGLE_LIBRARY' and l[2]=='1':
-          self.singlelib = True
-        elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_BLASLAPACK_UNDERSCORE' and l[2]=='1':
-          self.blaslapackmangling = 'underscore'
-        elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_BLASLAPACK_CAPS' and l[2]=='1':
-          self.blaslapackmangling = 'caps'
-        elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_HAVE_64BIT_BLAS_INDICES' and l[2]=='1':
-          self.blaslapackint64 = True
-        elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_HAVE_FORTRAN' and l[2]=='1':
-          self.fortran = True
-        elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_CLANGUAGE_CXX' and l[2]=='1':
-          self.language = 'c++'
-        elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_HAVE_CXX_DIALECT_CXX11' and l[2]=='1':
-          self.cxxdialectcxx11 = True
-        elif self.isinstall and len(l)==3 and l[0]=='#define' and l[1]=='PETSC_ARCH':
-          self.arch = l[2].strip('"')
-        else:
-          for p in ['elemental','hpddm','mkl','mkl_pardiso','scalapack','slepc']:
-            if len(l)==3 and l[0]=='#define' and l[1]=='PETSC_HAVE_'+p.upper() and l[2]=='1':
-              self.packages.append(p)
-      f.close()
+      with open(petscconf_h) as f:
+        for l in f.readlines():
+          l = l.split()
+          if len(l)==3 and l[0]=='#define' and l[1]=='PETSC_USE_64BIT_INDICES' and l[2]=='1':
+            self.ind64 = True
+          elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_HAVE_MPIUNI' and l[2]=='1':
+            self.mpiuni = True
+          elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_HAVE_MSMPI' and l[2]=='1':
+            self.msmpi = True
+          elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_USE_DEBUG' and l[2]=='1':
+            self.debug = True
+          elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_USE_SINGLE_LIBRARY' and l[2]=='1':
+            self.singlelib = True
+          elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_BLASLAPACK_UNDERSCORE' and l[2]=='1':
+            self.blaslapackmangling = 'underscore'
+          elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_BLASLAPACK_CAPS' and l[2]=='1':
+            self.blaslapackmangling = 'caps'
+          elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_HAVE_64BIT_BLAS_INDICES' and l[2]=='1':
+            self.blaslapackint64 = True
+          elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_HAVE_FORTRAN' and l[2]=='1':
+            self.fortran = True
+          elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_CLANGUAGE_CXX' and l[2]=='1':
+            self.language = 'c++'
+          elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_HAVE_CXX_DIALECT_CXX11' and l[2]=='1':
+            self.cxxdialectcxx11 = True
+          elif self.isinstall and len(l)==3 and l[0]=='#define' and l[1]=='PETSC_ARCH':
+            self.arch = l[2].strip('"')
+          else:
+            for p in ['elemental','hpddm','mkl_libs','mkl_includes','mkl_pardiso','scalapack','slepc']:
+              if len(l)==3 and l[0]=='#define' and l[1]=='PETSC_HAVE_'+p.upper() and l[2]=='1':
+                self.packages.append(p)
+      if 'mkl_libs' in self.packages and 'mkl_includes' in self.packages:
+        self.packages.remove('mkl_libs')
+        self.packages.remove('mkl_includes')
+        self.packages.append('mkl')
     except:
       if self.isinstall:
         self.log.Exit('Cannot process file ' + petscconf_h + ', maybe you forgot to set PETSC_ARCH')
@@ -219,7 +216,7 @@ class PETSc(package.Package):
     petsc.Check()
     if not petsc.havepackage:
       self.log.Exit('Unable to link with PETSc')
-    if slepc.isrepo and not petsc.isinstall and petsc.branch!='release' and slepc.branch!='release':
+    if slepc.isrepo and petsc.isrepo and not petsc.isinstall and petsc.branch!='release' and slepc.branch!='release':
       try:
         import dateutil.parser, datetime
         petscdate = dateutil.parser.parse(petsc.gitdate)
@@ -242,7 +239,7 @@ class PETSc(package.Package):
     try:
       (result, output) = self.RunCommand(self.fc+' --version',)
       import re
-      strmatch = re.match('GNU Fortran\s+\(.*\)\s+(\d+)\.(\d+)',output)
+      strmatch = re.match(r'GNU Fortran\s+\(.*\)\s+(\d+)\.(\d+)',output)
       if strmatch:
         VMAJOR,VMINOR = strmatch.groups()
         if (int(VMAJOR),int(VMINOR)) >= (10,0):
@@ -262,12 +259,18 @@ class PETSc(package.Package):
     return ' '.join(outflags)
 
   def getCXXFlags(self):
-    outflags = self.removeWarningFlags(self.cxx_flags.split())
-    return ' '.join(outflags)
+    if hasattr(self,'cxx_flags'):
+      outflags = self.removeWarningFlags(self.cxx_flags.split())
+      return ' '.join(outflags)
+    else:
+      return ''
 
   def getFFlags(self):
-    outflags = self.removeWarningFlags(self.fc_flags.split())
-    if self.isGfortran100plus():
-      outflags.append('-fallow-argument-mismatch')
-    return ' '.join(outflags)
+    if hasattr(self,'fc_flags'):
+      outflags = self.removeWarningFlags(self.fc_flags.split())
+      if self.isGfortran100plus():
+        outflags.append('-fallow-argument-mismatch')
+      return ' '.join(outflags)
+    else:
+      return ''
 
