@@ -1,7 +1,7 @@
 #
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #  SLEPc - Scalable Library for Eigenvalue Problem Computations
-#  Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
+#  Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 #
 #  This file is part of SLEPc.
 #  SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -73,7 +73,7 @@ class Slepc4py(package.Package):
     rule =  'slepc4pybuild:\n'
     rule += '\t@echo "*** Building slepc4py ***"\n'
     rule += '\t@${RM} -f '+os.path.join(confdir,'slepc4py.errorflg')+'\n'
-    rule += '\t@cd '+builddir+' && \\\n'
+    rule += '\t@cd '+builddir+' && ${RM} -rf build && \\\n'
     rule += '\t   %s ${PYTHON} setup.py build 2>&1 || \\\n' % envvars
     rule += '\t   (echo "**************************ERROR*************************************" && \\\n'
     rule += '\t   echo "Error building slepc4py." && \\\n'
@@ -105,8 +105,8 @@ class Slepc4py(package.Package):
       slepcrules.write('slepc4py-install:\n')
 
     rule =  'slepc4pytest:\n'
-    rule += '\t@echo "*** Testing slepc4py ***"\n'
-    rule += '\t@PYTHONPATH=%s:%s ${PYTHON} %s --verbose\n' % (destdir,pythonpath,os.path.join('src','binding','slepc4py','test','runtests.py'))
+    rule += '\t@echo "*** Testing slepc4py on ${PETSC4PY_NP} processes ***"\n'
+    rule += '\t@PYTHONPATH=%s:%s PETSC_OPTIONS="{PETSC_OPTIONS} -check_pointer_intensity 0 -error_output_stdout -malloc_dump ${PETSC_TEST_OPTIONS}" ${MPIEXEC} -n ${PETSC4PY_NP} ${PYTHON} %s --verbose\n' % (destdir,pythonpath,os.path.join('src','binding','slepc4py','test','runtests.py'))
     rule += '\t@echo "====================================="\n\n'
     slepcrules.write(rule)
 
