@@ -19,9 +19,9 @@ PetscErrorCode EPSComputeVectors(EPS eps)
 {
   PetscFunctionBegin;
   EPSCheckSolved(eps,1);
-  if (eps->state==EPS_STATE_SOLVED && eps->ops->computevectors) PetscCall((*eps->ops->computevectors)(eps));
+  if (eps->state==EPS_STATE_SOLVED) PetscTryTypeMethod(eps,computevectors);
   eps->state = EPS_STATE_EIGENVECTORS;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 #define SWAP(a,b,t) {t=a;a=b;b=t;}
@@ -39,8 +39,7 @@ static PetscErrorCode EPSComputeValues(EPS eps)
       PetscCall(STIsInjective(eps->st,&injective));
       if (injective) {
         /* one-to-one mapping: backtransform eigenvalues */
-        PetscCheck(eps->ops->backtransform,PetscObjectComm((PetscObject)eps),PETSC_ERR_PLIB,"Internal error, spectral transform should have a backtransform operation");
-        PetscCall((*eps->ops->backtransform)(eps));
+        PetscUseTypeMethod(eps,backtransform);
       } else {
         /* compute eigenvalues from Rayleigh quotient */
         PetscCall(DSGetDimensions(eps->ds,&n,NULL,NULL,NULL));
@@ -65,7 +64,7 @@ static PetscErrorCode EPSComputeValues(EPS eps)
           PetscCall(DSVectors(eps->ds,DS_MAT_X,NULL,NULL));
           PetscCall(DSGetMat(eps->ds,DS_MAT_X,&Z));
           PetscCall(BVMultInPlace(eps->V,Z,0,n));
-          PetscCall(MatDestroy(&Z));
+          PetscCall(DSRestoreMat(eps->ds,DS_MAT_X,&Z));
         }
         /* in case of STFILTER discard computed eigenvalues that lie outside the wanted interval */
         PetscCall(PetscObjectTypeCompare((PetscObject)eps->st,STFILTER,&isfilter));
@@ -86,13 +85,13 @@ static PetscErrorCode EPSComputeValues(EPS eps)
       /* eigenvalues already available as an output of the solver */
       break;
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    EPSSolve - Solves the eigensystem.
 
-   Collective on eps
+   Collective
 
    Input Parameter:
 .  eps - eigensolver context obtained from EPSCreate()
@@ -127,7 +126,7 @@ PetscErrorCode EPSSolve(EPS eps)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  if (eps->state>=EPS_STATE_SOLVED) PetscFunctionReturn(0);
+  if (eps->state>=EPS_STATE_SOLVED) PetscFunctionReturn(PETSC_SUCCESS);
   PetscCall(PetscLogEventBegin(EPS_Solve,eps,0,0,0));
 
   /* Call setup */
@@ -144,7 +143,7 @@ PetscErrorCode EPSSolve(EPS eps)
   PetscCall(RGViewFromOptions(eps->rg,NULL,"-rg_view"));
 
   /* Call solver */
-  PetscCall((*eps->ops->solve)(eps));
+  PetscUseTypeMethod(eps,solve);
   PetscCheck(eps->reason,PetscObjectComm((PetscObject)eps),PETSC_ERR_PLIB,"Internal error, solver returned without setting converged reason");
   eps->state = EPS_STATE_SOLVED;
 
@@ -196,7 +195,7 @@ PetscErrorCode EPSSolve(EPS eps)
     eps->nds = 0;
   }
   eps->nini = 0;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
@@ -229,7 +228,7 @@ PetscErrorCode EPSGetIterationNumber(EPS eps,PetscInt *its)
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidIntPointer(its,2);
   *its = eps->its;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
@@ -257,7 +256,7 @@ PetscErrorCode EPSGetConverged(EPS eps,PetscInt *nconv)
   PetscValidIntPointer(nconv,2);
   EPSCheckSolved(eps,1);
   *nconv = eps->nconv;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
@@ -296,14 +295,14 @@ PetscErrorCode EPSGetConvergedReason(EPS eps,EPSConvergedReason *reason)
   PetscValidIntPointer(reason,2);
   EPSCheckSolved(eps,1);
   *reason = eps->reason;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    EPSGetInvariantSubspace - Gets an orthonormal basis of the computed invariant
    subspace.
 
-   Not Collective, but vectors are shared by all processors that share the EPS
+   Collective
 
    Input Parameter:
 .  eps - the eigensolver context
@@ -353,14 +352,14 @@ PetscErrorCode EPSGetInvariantSubspace(EPS eps,Vec v[])
   }
   for (i=0;i<eps->nconv;i++) PetscCall(BVCopyVec(V,i,v[i]));
   if (eps->balance!=EPS_BALANCE_NONE && eps->D) PetscCall(BVDestroy(&V));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@C
    EPSGetEigenpair - Gets the i-th solution of the eigenproblem as computed by
    EPSSolve(). The solution consists in both the eigenvalue and the eigenvector.
 
-   Logically Collective on eps
+   Collective
 
    Input Parameters:
 +  eps - eigensolver context
@@ -405,7 +404,7 @@ PetscErrorCode EPSGetEigenpair(EPS eps,PetscInt i,PetscScalar *eigr,PetscScalar
   PetscCheck(i<eps->nconv,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index can be nconv-1 at most, see EPSGetConverged()");
   PetscCall(EPSGetEigenvalue(eps,i,eigr,eigi));
   if (Vr || Vi) PetscCall(EPSGetEigenvector(eps,i,Vr,Vi));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@C
@@ -451,13 +450,13 @@ PetscErrorCode EPSGetEigenvalue(EPS eps,PetscInt i,PetscScalar *eigr,PetscScalar
   if (eigr) *eigr = eps->eigr[k];
   if (eigi) *eigi = eps->eigi[k];
 #endif
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    EPSGetEigenvector - Gets the i-th right eigenvector as computed by EPSSolve().
 
-   Logically Collective on eps
+   Collective
 
    Input Parameters:
 +  eps - eigensolver context
@@ -503,13 +502,13 @@ PetscErrorCode EPSGetEigenvector(EPS eps,PetscInt i,Vec Vr,Vec Vi)
   PetscCall(EPSComputeVectors(eps));
   k = eps->perm[i];
   PetscCall(BV_GetEigenvector(eps->V,k,eps->eigi[k],Vr,Vi));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    EPSGetLeftEigenvector - Gets the i-th left eigenvector as computed by EPSSolve().
 
-   Logically Collective on eps
+   Collective
 
    Input Parameters:
 +  eps - eigensolver context
@@ -555,7 +554,7 @@ PetscErrorCode EPSGetLeftEigenvector(EPS eps,PetscInt i,Vec Wr,Vec Wi)
   PetscCall(EPSComputeVectors(eps));
   k = eps->perm[i];
   PetscCall(BV_GetEigenvector(eps->W,k,eps->eigi[k],Wr,Wi));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
@@ -589,7 +588,7 @@ PetscErrorCode EPSGetErrorEstimate(EPS eps,PetscInt i,PetscReal *errest)
   PetscCheck(i>=0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index cannot be negative");
   PetscCheck(i<eps->nconv,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index can be nconv-1 at most, see EPSGetConverged()");
   *errest = eps->errest[eps->perm[i]];
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*
@@ -653,14 +652,14 @@ PetscErrorCode EPSComputeResidualNorm_Private(EPS eps,PetscBool trans,PetscScala
     *norm = SlepcAbsEigenvalue(nr,ni);
   }
 #endif
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    EPSComputeError - Computes the error (based on the residual norm) associated
    with the i-th computed eigenpair.
 
-   Collective on eps
+   Collective
 
    Input Parameters:
 +  eps  - the eigensolver context
@@ -750,14 +749,14 @@ PetscErrorCode EPSComputeError(EPS eps,PetscInt i,EPSErrorType type,PetscReal *e
     default:
       SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Invalid error type");
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*
    EPSGetStartVector - Generate a suitable vector to be used as the starting vector
    for the recurrence that builds the right subspace.
 
-   Collective on eps
+   Collective
 
    Input Parameters:
 +  eps - the eigensolver context
@@ -808,7 +807,7 @@ PetscErrorCode EPSGetStartVector(EPS eps,PetscInt i,PetscBool *breakdown)
     PetscCheck(!i,PetscObjectComm((PetscObject)eps),PETSC_ERR_CONV_FAILED,"Unable to generate more start vectors");
   }
   PetscCall(BVScaleColumn(eps->V,i,1.0/norm));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*
@@ -835,5 +834,5 @@ PetscErrorCode EPSGetLeftStartVector(EPS eps,PetscInt i,PetscBool *breakdown)
     SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_CONV_FAILED,"Unable to generate more left start vectors");
   }
   PetscCall(BVScaleColumn(eps->W,i,1.0/norm));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
