@@ -62,14 +62,14 @@ static PetscErrorCode SlepcMatDenseSqrt(PetscBLASInt n,PetscScalar *T,PetscBLASI
 #endif
       /* solve Sylvester equation of order si x sj */
       r = j-i-si;
-      if (r) PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&si,&sj,&r,&mone,T+i+(i+si)*ld,&ld,T+i+si+j*ld,&ld,&one,T+i+j*ld,&ld));
-      PetscStackCallBLAS("LAPACKtrsyl",LAPACKtrsyl_("N","N",&ione,&si,&sj,T+i+i*ld,&ld,T+j+j*ld,&ld,T+i+j*ld,&ld,&scal,&info));
+      if (r) PetscCallBLAS("BLASgemm",BLASgemm_("N","N",&si,&sj,&r,&mone,T+i+(i+si)*ld,&ld,T+i+si+j*ld,&ld,&one,T+i+j*ld,&ld));
+      PetscCallBLAS("LAPACKtrsyl",LAPACKtrsyl_("N","N",&ione,&si,&sj,T+i+i*ld,&ld,T+j+j*ld,&ld,T+i+j*ld,&ld,&scal,&info));
       SlepcCheckLapackInfo("trsyl",info);
       PetscCheck(scal==1.0,PETSC_COMM_SELF,PETSC_ERR_SUP,"Current implementation cannot handle scale factor %g",(double)scal);
     }
     if (sj==2) j++;
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 #define BLOCKSIZE 64
@@ -100,10 +100,10 @@ PetscErrorCode FNSqrtmSchur(FN fn,PetscBLASInt n,PetscScalar *T,PetscBLASInt ld,
   /* compute Schur decomposition A*Q = Q*T */
 #if !defined(PETSC_USE_COMPLEX)
   PetscCall(PetscMalloc7(m,&wr,m,&wi,m*k,&W,m*m,&Q,lwork,&work,nblk,&s,nblk,&p));
-  PetscStackCallBLAS("LAPACKgees",LAPACKgees_("V","N",NULL,&n,T,&ld,&sdim,wr,wi,Q,&ld,work,&lwork,NULL,&info));
+  PetscCallBLAS("LAPACKgees",LAPACKgees_("V","N",NULL,&n,T,&ld,&sdim,wr,wi,Q,&ld,work,&lwork,NULL,&info));
 #else
   PetscCall(PetscMalloc7(m,&wr,m,&rwork,m*k,&W,m*m,&Q,lwork,&work,nblk,&s,nblk,&p));
-  PetscStackCallBLAS("LAPACKgees",LAPACKgees_("V","N",NULL,&n,T,&ld,&sdim,wr,Q,&ld,work,&lwork,rwork,NULL,&info));
+  PetscCallBLAS("LAPACKgees",LAPACKgees_("V","N",NULL,&n,T,&ld,&sdim,wr,Q,&ld,work,&lwork,rwork,NULL,&info));
 #endif
   SlepcCheckLapackInfo("gees",info);
 
@@ -127,16 +127,16 @@ PetscErrorCode FNSqrtmSchur(FN fn,PetscBLASInt n,PetscScalar *T,PetscBLASInt ld,
     for (i=j-1;i>=0;i--) {
       /* solve Sylvester equation for block (i,j) */
       r = p[j]-p[i]-s[i];
-      if (r) PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",s+i,s+j,&r,&mone,T+p[i]+(p[i]+s[i])*ld,&ld,T+p[i]+s[i]+p[j]*ld,&ld,&one,T+p[i]+p[j]*ld,&ld));
-      PetscStackCallBLAS("LAPACKtrsyl",LAPACKtrsyl_("N","N",&ione,s+i,s+j,T+p[i]+p[i]*ld,&ld,T+p[j]+p[j]*ld,&ld,T+p[i]+p[j]*ld,&ld,&scal,&info));
+      if (r) PetscCallBLAS("BLASgemm",BLASgemm_("N","N",s+i,s+j,&r,&mone,T+p[i]+(p[i]+s[i])*ld,&ld,T+p[i]+s[i]+p[j]*ld,&ld,&one,T+p[i]+p[j]*ld,&ld));
+      PetscCallBLAS("LAPACKtrsyl",LAPACKtrsyl_("N","N",&ione,s+i,s+j,T+p[i]+p[i]*ld,&ld,T+p[j]+p[j]*ld,&ld,T+p[i]+p[j]*ld,&ld,&scal,&info));
       SlepcCheckLapackInfo("trsyl",info);
       PetscCheck(scal==1.0,PETSC_COMM_SELF,PETSC_ERR_SUP,"Current implementation cannot handle scale factor %g",(double)scal);
     }
   }
 
   /* backtransform B = Q*T*Q' */
-  PetscStackCallBLAS("BLASgemm",BLASgemm_("N","C",&n,&k,&n,&one,T,&ld,Q,&ld,&zero,W,&ld));
-  PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&k,&n,&one,Q,&ld,W,&ld,&zero,T,&ld));
+  PetscCallBLAS("BLASgemm",BLASgemm_("N","C",&n,&k,&n,&one,T,&ld,Q,&ld,&zero,W,&ld));
+  PetscCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&k,&n,&one,Q,&ld,W,&ld,&zero,T,&ld));
 
   /* flop count: Schur decomposition, triangular square root, and backtransform */
   PetscCall(PetscLogFlops(25.0*n*n*n+n*n*n/3.0+4.0*n*n*k));
@@ -146,7 +146,7 @@ PetscErrorCode FNSqrtmSchur(FN fn,PetscBLASInt n,PetscScalar *T,PetscBLASInt ld,
 #else
   PetscCall(PetscFree7(wr,rwork,W,Q,work,s,p));
 #endif
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 #define DBMAXIT 25
@@ -163,16 +163,17 @@ PetscErrorCode FNSqrtmDenmanBeavers(FN fn,PetscBLASInt n,PetscScalar *T,PetscBLA
   PetscReal          tol,Mres=0.0,detM,g,reldiff,fnormdiff,fnormT,rwork[1];
   PetscBLASInt       N,i,it,*piv=NULL,info,query=-1,lwork;
   const PetscBLASInt one=1;
-  PetscBool          converged=PETSC_FALSE,scale=PETSC_FALSE;
+  PetscBool          converged=PETSC_FALSE,scale;
   unsigned int       ftz;
 
   PetscFunctionBegin;
   N = n*n;
   tol = PetscSqrtReal((PetscReal)n)*PETSC_MACHINE_EPSILON/2;
+  scale = PetscDefined(USE_REAL_SINGLE)? PETSC_FALSE: PETSC_TRUE;
   PetscCall(SlepcSetFlushToZero(&ftz));
 
   /* query work size */
-  PetscStackCallBLAS("LAPACKgetri",LAPACKgetri_(&n,M,&ld,piv,&work1,&query,&info));
+  PetscCallBLAS("LAPACKgetri",LAPACKgetri_(&n,M,&ld,piv,&work1,&query,&info));
   PetscCall(PetscBLASIntCast((PetscInt)PetscRealPart(work1),&lwork));
   PetscCall(PetscMalloc5(lwork,&work,n,&piv,n*n,&Told,n*n,&M,n*n,&invM));
   PetscCall(PetscArraycpy(M,T,n*n));
@@ -186,34 +187,34 @@ PetscErrorCode FNSqrtmDenmanBeavers(FN fn,PetscBLASInt n,PetscScalar *T,PetscBLA
 
     if (scale) {  /* g = (abs(det(M)))^(-1/(2*n)) */
       PetscCall(PetscArraycpy(invM,M,n*n));
-      PetscStackCallBLAS("LAPACKgetrf",LAPACKgetrf_(&n,&n,invM,&ld,piv,&info));
+      PetscCallBLAS("LAPACKgetrf",LAPACKgetrf_(&n,&n,invM,&ld,piv,&info));
       SlepcCheckLapackInfo("getrf",info);
       prod = invM[0];
       for (i=1;i<n;i++) prod *= invM[i+i*ld];
       detM = PetscAbsScalar(prod);
-      g = PetscPowReal(detM,-1.0/(2.0*n));
+      g = (detM>PETSC_MAX_REAL)? 0.5: PetscPowReal(detM,-1.0/(2.0*n));
       alpha = g;
-      PetscStackCallBLAS("BLASscal",BLASscal_(&N,&alpha,T,&one));
+      PetscCallBLAS("BLASscal",BLASscal_(&N,&alpha,T,&one));
       alpha = g*g;
-      PetscStackCallBLAS("BLASscal",BLASscal_(&N,&alpha,M,&one));
+      PetscCallBLAS("BLASscal",BLASscal_(&N,&alpha,M,&one));
       PetscCall(PetscLogFlops(2.0*n*n*n/3.0+2.0*n*n));
     }
 
     PetscCall(PetscArraycpy(Told,T,n*n));
     PetscCall(PetscArraycpy(invM,M,n*n));
 
-    PetscStackCallBLAS("LAPACKgetrf",LAPACKgetrf_(&n,&n,invM,&ld,piv,&info));
+    PetscCallBLAS("LAPACKgetrf",LAPACKgetrf_(&n,&n,invM,&ld,piv,&info));
     SlepcCheckLapackInfo("getrf",info);
-    PetscStackCallBLAS("LAPACKgetri",LAPACKgetri_(&n,invM,&ld,piv,work,&lwork,&info));
+    PetscCallBLAS("LAPACKgetri",LAPACKgetri_(&n,invM,&ld,piv,work,&lwork,&info));
     SlepcCheckLapackInfo("getri",info);
     PetscCall(PetscLogFlops(2.0*n*n*n/3.0+4.0*n*n*n/3.0));
 
     for (i=0;i<n;i++) invM[i+i*ld] += 1.0;
-    PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&spfive,Told,&ld,invM,&ld,&szero,T,&ld));
+    PetscCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&spfive,Told,&ld,invM,&ld,&szero,T,&ld));
     for (i=0;i<n;i++) invM[i+i*ld] -= 1.0;
 
-    PetscStackCallBLAS("BLASaxpy",BLASaxpy_(&N,&sone,invM,&one,M,&one));
-    PetscStackCallBLAS("BLASscal",BLASscal_(&N,&sp25,M,&one));
+    PetscCallBLAS("BLASaxpy",BLASaxpy_(&N,&sone,invM,&one,M,&one));
+    PetscCallBLAS("BLASscal",BLASscal_(&N,&sp25,M,&one));
     for (i=0;i<n;i++) M[i+i*ld] -= 0.5;
     PetscCall(PetscLogFlops(2.0*n*n*n+2.0*n*n));
 
@@ -222,7 +223,7 @@ PetscErrorCode FNSqrtmDenmanBeavers(FN fn,PetscBLASInt n,PetscScalar *T,PetscBLA
 
     if (scale) {
       /* reldiff = norm(T - Told,'fro')/norm(T,'fro') */
-      PetscStackCallBLAS("BLASaxpy",BLASaxpy_(&N,&smone,T,&one,Told,&one));
+      PetscCallBLAS("BLASaxpy",BLASaxpy_(&N,&smone,T,&one,Told,&one));
       fnormdiff = LAPACKlange_("F",&n,&n,Told,&n,rwork);
       fnormT = LAPACKlange_("F",&n,&n,T,&n,rwork);
       PetscCall(PetscLogFlops(7.0*n*n));
@@ -237,7 +238,7 @@ PetscErrorCode FNSqrtmDenmanBeavers(FN fn,PetscBLASInt n,PetscScalar *T,PetscBLA
   PetscCheck(Mres<=tol,PETSC_COMM_SELF,PETSC_ERR_LIB,"SQRTM not converged after %d iterations",DBMAXIT);
   PetscCall(PetscFree5(work,piv,Told,M,invM));
   PetscCall(SlepcResetFlushToZero(&ftz));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 #define NSMAXIT 50
@@ -262,12 +263,12 @@ PetscErrorCode FNSqrtmNewtonSchulz(FN fn,PetscBLASInt n,PetscScalar *A,PetscBLAS
 
   PetscCall(PetscMalloc4(N,&Yold,N,&Z,N,&Zold,N,&M));
 
-  /* scale A so that ||I-A|| < 1 */
+  /* scale */
   PetscCall(PetscArraycpy(Z,A,N));
   for (i=0;i<n;i++) Z[i+i*ld] -= 1.0;
   nrm = LAPACKlange_("fro",&n,&n,Z,&n,rwork);
   sqrtnrm = PetscSqrtReal(nrm);
-  PetscStackCallBLAS("LAPACKlascl",LAPACKlascl_("G",&zero,&zero,&nrm,&done,&N,&one,A,&N,&info));
+  PetscCallBLAS("LAPACKlascl",LAPACKlascl_("G",&zero,&zero,&nrm,&done,&N,&one,A,&N,&info));
   SlepcCheckLapackInfo("lascl",info);
   tol *= nrm;
   PetscCall(PetscInfo(fn,"||I-A||_F = %g, new tol: %g\n",(double)nrm,(double)tol));
@@ -285,14 +286,14 @@ PetscErrorCode FNSqrtmNewtonSchulz(FN fn,PetscBLASInt n,PetscScalar *A,PetscBLAS
     /* M = (3*I-Zold*Yold) */
     PetscCall(PetscArrayzero(M,N));
     for (i=0;i<n;i++) M[i+i*ld] = sthree;
-    PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&smone,Zold,&ld,Yold,&ld,&sone,M,&ld));
+    PetscCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&smone,Zold,&ld,Yold,&ld,&sone,M,&ld));
 
     /* Y = (1/2)*Yold*M, Z = (1/2)*M*Zold */
-    PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&spfive,Yold,&ld,M,&ld,&szero,Y,&ld));
-    PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&spfive,M,&ld,Zold,&ld,&szero,Z,&ld));
+    PetscCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&spfive,Yold,&ld,M,&ld,&szero,Y,&ld));
+    PetscCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&spfive,M,&ld,Zold,&ld,&szero,Z,&ld));
 
     /* reldiff = norm(Y-Yold,'fro')/norm(Y,'fro') */
-    PetscStackCallBLAS("BLASaxpy",BLASaxpy_(&N,&smone,Y,&one,Yold,&one));
+    PetscCallBLAS("BLASaxpy",BLASaxpy_(&N,&smone,Y,&one,Yold,&one));
     Yres = LAPACKlange_("fro",&n,&n,Yold,&n,rwork);
     PetscCheck(!PetscIsNanReal(Yres),PETSC_COMM_SELF,PETSC_ERR_FP,"The computed norm is not-a-number");
     if (Yres<=tol) converged = PETSC_TRUE;
@@ -306,13 +307,13 @@ PetscErrorCode FNSqrtmNewtonSchulz(FN fn,PetscBLASInt n,PetscScalar *A,PetscBLAS
   /* undo scaling */
   if (inv) {
     PetscCall(PetscArraycpy(A,Z,N));
-    PetscStackCallBLAS("LAPACKlascl",LAPACKlascl_("G",&zero,&zero,&sqrtnrm,&done,&N,&one,A,&N,&info));
-  } else PetscStackCallBLAS("LAPACKlascl",LAPACKlascl_("G",&zero,&zero,&done,&sqrtnrm,&N,&one,A,&N,&info));
+    PetscCallBLAS("LAPACKlascl",LAPACKlascl_("G",&zero,&zero,&sqrtnrm,&done,&N,&one,A,&N,&info));
+  } else PetscCallBLAS("LAPACKlascl",LAPACKlascl_("G",&zero,&zero,&done,&sqrtnrm,&N,&one,A,&N,&info));
   SlepcCheckLapackInfo("lascl",info);
 
   PetscCall(PetscFree4(Yold,Z,Zold,M));
   PetscCall(SlepcResetFlushToZero(&ftz));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 #if defined(PETSC_HAVE_CUDA)
@@ -321,18 +322,17 @@ PetscErrorCode FNSqrtmNewtonSchulz(FN fn,PetscBLASInt n,PetscScalar *A,PetscBLAS
 
 /*
  * Matrix square root by Newton-Schulz iteration. CUDA version.
- * Computes the principal square root of the matrix T using the
- * Newton-Schulz iteration. T is overwritten with sqrtm(T).
+ * Computes the principal square root of the matrix A using the
+ * Newton-Schulz iteration. A is overwritten with sqrtm(A).
  */
-PetscErrorCode FNSqrtmNewtonSchulz_CUDA(FN fn,PetscBLASInt n,PetscScalar *A,PetscBLASInt ld,PetscBool inv)
+PetscErrorCode FNSqrtmNewtonSchulz_CUDA(FN fn,PetscBLASInt n,PetscScalar *d_A,PetscBLASInt ld,PetscBool inv)
 {
-  PetscScalar        *d_A,*d_Yold,*d_Z,*d_Zold,*d_M;
-  PetscReal          nrm,sqrtnrm;
+  PetscScalar        *d_Yold,*d_Z,*d_Zold,*d_M,alpha;
+  PetscReal          nrm,sqrtnrm,tol,Yres=0.0;
   const PetscScalar  szero=0.0,sone=1.0,smone=-1.0,spfive=0.5,sthree=3.0;
-  PetscReal          tol,Yres=0.0,alpha;
   PetscInt           it;
   PetscBLASInt       N;
-  const PetscBLASInt one=1,zero=0;
+  const PetscBLASInt one=1;
   PetscBool          converged=PETSC_FALSE;
   cublasHandle_t     cublasv2handle;
 
@@ -342,7 +342,6 @@ PetscErrorCode FNSqrtmNewtonSchulz_CUDA(FN fn,PetscBLASInt n,PetscScalar *A,Pets
   N = n*n;
   tol = PetscSqrtReal((PetscReal)n)*PETSC_MACHINE_EPSILON/2;
 
-  PetscCallCUDA(cudaMalloc((void **)&d_A,sizeof(PetscScalar)*N));
   PetscCallCUDA(cudaMalloc((void **)&d_Yold,sizeof(PetscScalar)*N));
   PetscCallCUDA(cudaMalloc((void **)&d_Z,sizeof(PetscScalar)*N));
   PetscCallCUDA(cudaMalloc((void **)&d_Zold,sizeof(PetscScalar)*N));
@@ -350,13 +349,11 @@ PetscErrorCode FNSqrtmNewtonSchulz_CUDA(FN fn,PetscBLASInt n,PetscScalar *A,Pets
 
   PetscCall(PetscLogGpuTimeBegin());
 
-  /* Y = A; */
-  PetscCallCUDA(cudaMemcpy(d_A,A,sizeof(PetscScalar)*N,cudaMemcpyHostToDevice));
   /* Z = I; */
-  PetscCallCUDA(cudaMemset(d_Z,zero,sizeof(PetscScalar)*N));
+  PetscCallCUDA(cudaMemset(d_Z,0,sizeof(PetscScalar)*N));
   PetscCall(set_diagonal(n,d_Z,ld,sone));
 
-  /* scale A so that ||I-A|| < 1 */
+  /* scale */
   PetscCallCUBLAS(cublasXaxpy(cublasv2handle,N,&smone,d_A,one,d_Z,one));
   PetscCallCUBLAS(cublasXnrm2(cublasv2handle,N,d_Z,one,&nrm));
   sqrtnrm = PetscSqrtReal(nrm);
@@ -367,7 +364,7 @@ PetscErrorCode FNSqrtmNewtonSchulz_CUDA(FN fn,PetscBLASInt n,PetscScalar *A,Pets
   PetscCall(PetscLogGpuFlops(2.0*n*n));
 
   /* Z = I; */
-  PetscCallCUDA(cudaMemset(d_Z,zero,sizeof(PetscScalar)*N));
+  PetscCallCUDA(cudaMemset(d_Z,0,sizeof(PetscScalar)*N));
   PetscCall(set_diagonal(n,d_Z,ld,sone));
 
   for (it=0;it<NSMAXIT && !converged;it++) {
@@ -376,7 +373,7 @@ PetscErrorCode FNSqrtmNewtonSchulz_CUDA(FN fn,PetscBLASInt n,PetscScalar *A,Pets
     PetscCallCUDA(cudaMemcpy(d_Zold,d_Z,sizeof(PetscScalar)*N,cudaMemcpyDeviceToDevice));
 
     /* M = (3*I - Zold*Yold) */
-    PetscCallCUDA(cudaMemset(d_M,zero,sizeof(PetscScalar)*N));
+    PetscCallCUDA(cudaMemset(d_M,0,sizeof(PetscScalar)*N));
     PetscCall(set_diagonal(n,d_M,ld,sthree));
     PetscCallCUBLAS(cublasXgemm(cublasv2handle,CUBLAS_OP_N,CUBLAS_OP_N,n,n,n,&smone,d_Zold,ld,d_Yold,ld,&sone,d_M,ld));
 
@@ -398,21 +395,20 @@ PetscErrorCode FNSqrtmNewtonSchulz_CUDA(FN fn,PetscBLASInt n,PetscScalar *A,Pets
 
   /* undo scaling */
   if (inv) {
-    sqrtnrm = 1.0/sqrtnrm;
-    PetscCallCUBLAS(cublasXscal(cublasv2handle,N,&sqrtnrm,d_Z,one));
-    PetscCallCUDA(cudaMemcpy(A,d_Z,sizeof(PetscScalar)*N,cudaMemcpyDeviceToHost));
+    alpha = 1.0/sqrtnrm;
+    PetscCallCUBLAS(cublasXscal(cublasv2handle,N,&alpha,d_Z,one));
+    PetscCallCUDA(cudaMemcpy(d_A,d_Z,sizeof(PetscScalar)*N,cudaMemcpyDeviceToDevice));
   } else {
-    PetscCallCUBLAS(cublasXscal(cublasv2handle,N,&sqrtnrm,d_A,one));
-    PetscCallCUDA(cudaMemcpy(A,d_A,sizeof(PetscScalar)*N,cudaMemcpyDeviceToHost));
+    alpha = sqrtnrm;
+    PetscCallCUBLAS(cublasXscal(cublasv2handle,N,&alpha,d_A,one));
   }
 
   PetscCall(PetscLogGpuTimeEnd());
-  PetscCallCUDA(cudaFree(d_A));
   PetscCallCUDA(cudaFree(d_Yold));
   PetscCallCUDA(cudaFree(d_Z));
   PetscCallCUDA(cudaFree(d_Zold));
   PetscCallCUDA(cudaFree(d_M));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 #if defined(PETSC_HAVE_MAGMA)
@@ -423,20 +419,21 @@ PetscErrorCode FNSqrtmNewtonSchulz_CUDA(FN fn,PetscBLASInt n,PetscScalar *A,Pets
  * Computes the principal square root of the matrix T using the product form
  * of the Denman-Beavers iteration. T is overwritten with sqrtm(T).
  */
-PetscErrorCode FNSqrtmDenmanBeavers_CUDAm(FN fn,PetscBLASInt n,PetscScalar *T,PetscBLASInt ld,PetscBool inv)
+PetscErrorCode FNSqrtmDenmanBeavers_CUDAm(FN fn,PetscBLASInt n,PetscScalar *d_T,PetscBLASInt ld,PetscBool inv)
 {
-  PetscScalar    *d_T,*d_Told,*d_M,*d_invM,*d_work,zero=0.0,sone=1.0,smone=-1.0,spfive=0.5,sneg_pfive=-0.5,sp25=0.25,alpha;
-  PetscReal      tol,Mres=0.0,detM,g,reldiff,fnormdiff,fnormT,prod;
-  PetscInt       i,it,lwork,nb;
+  PetscScalar    *d_Told,*d_M,*d_invM,*d_work,prod,szero=0.0,sone=1.0,smone=-1.0,spfive=0.5,sneg_pfive=-0.5,sp25=0.25,alpha;
+  PetscReal      tol,Mres=0.0,detM,g,reldiff,fnormdiff,fnormT;
+  PetscInt       it,lwork,nb;
   PetscBLASInt   N,one=1,*piv=NULL;
-  PetscBool      converged=PETSC_FALSE,scale=PETSC_FALSE;
+  PetscBool      converged=PETSC_FALSE,scale;
   cublasHandle_t cublasv2handle;
 
   PetscFunctionBegin;
   PetscCall(PetscDeviceInitialize(PETSC_DEVICE_CUDA)); /* For CUDA event timers */
   PetscCall(PetscCUBLASGetHandle(&cublasv2handle));
-  magma_init();
+  PetscCall(SlepcMagmaInit());
   N = n*n;
+  scale = PetscDefined(USE_REAL_SINGLE)? PETSC_FALSE: PETSC_TRUE;
   tol = PetscSqrtReal((PetscReal)n)*PETSC_MACHINE_EPSILON/2;
 
   /* query work size */
@@ -444,31 +441,25 @@ PetscErrorCode FNSqrtmDenmanBeavers_CUDAm(FN fn,PetscBLASInt n,PetscScalar *T,Pe
   lwork = nb*n;
   PetscCall(PetscMalloc1(n,&piv));
   PetscCallCUDA(cudaMalloc((void **)&d_work,sizeof(PetscScalar)*lwork));
-  PetscCallCUDA(cudaMalloc((void **)&d_T,sizeof(PetscScalar)*N));
   PetscCallCUDA(cudaMalloc((void **)&d_Told,sizeof(PetscScalar)*N));
   PetscCallCUDA(cudaMalloc((void **)&d_M,sizeof(PetscScalar)*N));
   PetscCallCUDA(cudaMalloc((void **)&d_invM,sizeof(PetscScalar)*N));
 
   PetscCall(PetscLogGpuTimeBegin());
-  PetscCallCUDA(cudaMemcpy(d_M,T,sizeof(PetscScalar)*N,cudaMemcpyHostToDevice));
+  PetscCallCUDA(cudaMemcpy(d_M,d_T,sizeof(PetscScalar)*N,cudaMemcpyDeviceToDevice));
   if (inv) {  /* start recurrence with I instead of A */
-    PetscCallCUDA(cudaMemset(d_T,zero,sizeof(PetscScalar)*N));
+    PetscCallCUDA(cudaMemset(d_T,0,sizeof(PetscScalar)*N));
     PetscCall(set_diagonal(n,d_T,ld,1.0));
-  } else PetscCallCUDA(cudaMemcpy(d_T,T,sizeof(PetscScalar)*N,cudaMemcpyHostToDevice));
+  }
 
   for (it=0;it<DBMAXIT && !converged;it++) {
 
     if (scale) { /* g = (abs(det(M)))^(-1/(2*n)); */
       PetscCallCUDA(cudaMemcpy(d_invM,d_M,sizeof(PetscScalar)*N,cudaMemcpyDeviceToDevice));
       PetscCallMAGMA(magma_xgetrf_gpu,n,n,d_invM,ld,piv);
-
-      /* XXX pending */
-//      PetscCall(mult_diagonal(d_invM,n,ld,&detM));
-      PetscCallCUDA(cudaMemcpy(T,d_invM,sizeof(PetscScalar)*N,cudaMemcpyDeviceToHost));
-      prod = T[0];
-      for (i=1;i<n;i++) { prod *= T[i+i*ld]; }
-      detM = PetscAbsReal(prod);
-      g = PetscPowReal(detM,-1.0/(2.0*n));
+      PetscCall(mult_diagonal(n,d_invM,ld,&prod));
+      detM = PetscAbsScalar(prod);
+      g = (detM>PETSC_MAX_REAL)? 0.5: PetscPowReal(detM,-1.0/(2.0*n));
       alpha = g;
       PetscCallCUBLAS(cublasXscal(cublasv2handle,N,&alpha,d_T,one));
       alpha = g*g;
@@ -484,7 +475,7 @@ PetscErrorCode FNSqrtmDenmanBeavers_CUDAm(FN fn,PetscBLASInt n,PetscScalar *T,Pe
     PetscCall(PetscLogGpuFlops(2.0*n*n*n/3.0+4.0*n*n*n/3.0));
 
     PetscCall(shift_diagonal(n,d_invM,ld,sone));
-    PetscCallCUBLAS(cublasXgemm(cublasv2handle,CUBLAS_OP_N,CUBLAS_OP_N,n,n,n,&spfive,d_Told,ld,d_invM,ld,&zero,d_T,ld));
+    PetscCallCUBLAS(cublasXgemm(cublasv2handle,CUBLAS_OP_N,CUBLAS_OP_N,n,n,n,&spfive,d_Told,ld,d_invM,ld,&szero,d_T,ld));
     PetscCall(shift_diagonal(n,d_invM,ld,smone));
 
     PetscCallCUBLAS(cublasXaxpy(cublasv2handle,N,&sone,d_invM,one,d_M,one));
@@ -496,7 +487,7 @@ PetscErrorCode FNSqrtmDenmanBeavers_CUDAm(FN fn,PetscBLASInt n,PetscScalar *T,Pe
     PetscCall(shift_diagonal(n,d_M,ld,sone));
 
     if (scale) {
-      // reldiff = norm(T - Told,'fro')/norm(T,'fro');
+      /* reldiff = norm(T - Told,'fro')/norm(T,'fro'); */
       PetscCallCUBLAS(cublasXaxpy(cublasv2handle,N,&smone,d_T,one,d_Told,one));
       PetscCallCUBLAS(cublasXnrm2(cublasv2handle,N,d_Told,one,&fnormdiff));
       PetscCallCUBLAS(cublasXnrm2(cublasv2handle,N,d_T,one,&fnormT));
@@ -511,16 +502,13 @@ PetscErrorCode FNSqrtmDenmanBeavers_CUDAm(FN fn,PetscBLASInt n,PetscScalar *T,Pe
   }
 
   PetscCheck(Mres<=tol,PETSC_COMM_SELF,PETSC_ERR_LIB,"SQRTM not converged after %d iterations", DBMAXIT);
-  PetscCallCUDA(cudaMemcpy(T,d_T,sizeof(PetscScalar)*N,cudaMemcpyDeviceToHost));
   PetscCall(PetscLogGpuTimeEnd());
   PetscCall(PetscFree(piv));
   PetscCallCUDA(cudaFree(d_work));
-  PetscCallCUDA(cudaFree(d_T));
   PetscCallCUDA(cudaFree(d_Told));
   PetscCallCUDA(cudaFree(d_M));
   PetscCallCUDA(cudaFree(d_invM));
-  magma_finalize();
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 #endif /* PETSC_HAVE_MAGMA */
 
@@ -558,7 +546,7 @@ static PetscErrorCode SlepcNormEst1(PetscBLASInt n,PetscScalar *A,PetscInt m,Pet
   while (1) {
     it++;
     for (j=0;j<m;j++) {  /* Y = A^m*X */
-      PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&t,&n,&sone,A,&n,X,&n,&szero,Y,&n));
+      PetscCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&t,&n,&sone,A,&n,X,&n,&szero,Y,&n));
       if (j<m-1) SWAP(X,Y,aux);
     }
     for (j=0;j<t;j++) {  /* vals[j] = norm(Y(:,j),1) */
@@ -582,7 +570,7 @@ static PetscErrorCode SlepcNormEst1(PetscBLASInt n,PetscScalar *A,PetscInt m,Pet
       S[i] = (PetscRealPart(Y[i]) < 0.0)? -1.0: 1.0;
     }
     for (j=0;j<m;j++) {  /* Z = (A^T)^m*S */
-      PetscStackCallBLAS("BLASgemm",BLASgemm_("C","N",&n,&t,&n,&sone,A,&n,S,&n,&szero,Z,&n));
+      PetscCallBLAS("BLASgemm",BLASgemm_("C","N",&n,&t,&n,&sone,A,&n,S,&n,&szero,Z,&n));
       if (j<m-1) SWAP(S,Z,aux);
     }
     maxzval[0] = -1; maxzval[1] = -1;
@@ -604,7 +592,7 @@ static PetscErrorCode SlepcNormEst1(PetscBLASInt n,PetscScalar *A,PetscInt m,Pet
   *nrm = est;
   /* Flop count is roughly (it * 2*m * t*gemv) = 4*its*m*t*n*n */
   PetscCall(PetscLogFlops(4.0*it*m*t*n*n));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 #define SMALLN 100
@@ -625,9 +613,9 @@ PetscErrorCode SlepcNormAm(PetscBLASInt n,PetscScalar *A,PetscInt m,PetscScalar
       *nrm = LAPACKlange_("O",&n,&n,A,&n,rwork);
       PetscCall(PetscLogFlops(1.0*n*n));
     } else {  /* m>=2 */
-      PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&sone,A,&n,A,&n,&szero,v,&n));
+      PetscCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&sone,A,&n,A,&n,&szero,v,&n));
       for (j=0;j<m-2;j++) {
-        PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&sone,A,&n,v,&n,&szero,w,&n));
+        PetscCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&sone,A,&n,v,&n,&szero,w,&n));
         SWAP(v,w,aux);
       }
       *nrm = LAPACKlange_("O",&n,&n,v,&n,rwork);
@@ -644,7 +632,7 @@ PetscErrorCode SlepcNormAm(PetscBLASInt n,PetscScalar *A,PetscInt m,PetscScalar
     if (isrealpos) {   /* for positive matrices only */
       for (i=0;i<n;i++) v[i] = 1.0;
       for (j=0;j<m;j++) {  /* w = A'*v */
-        PetscStackCallBLAS("BLASgemv",BLASgemv_("C",&n,&n,&sone,A,&n,v,&one,&szero,w,&one));
+        PetscCallBLAS("BLASgemv",BLASgemv_("C",&n,&n,&sone,A,&n,v,&one,&szero,w,&one));
         SWAP(v,w,aux);
       }
       PetscCall(PetscLogFlops(2.0*n*n*m));
@@ -652,5 +640,5 @@ PetscErrorCode SlepcNormAm(PetscBLASInt n,PetscScalar *A,PetscInt m,PetscScalar
       for (i=0;i<n;i++) if ((tmp = PetscAbsScalar(v[i])) > *nrm) *nrm = tmp;   /* norm(v,inf) */
     } else PetscCall(SlepcNormEst1(n,A,m,work,rand,nrm));
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
