@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -19,7 +19,7 @@ SLEPC_EXTERN PetscErrorCode PCCreate_HPDDM(PC);
 /*@C
     SlepcGetVersion - Gets the SLEPc version information in a string.
 
-    Not collective
+    Not Collective
 
     Input Parameter:
 .   len - length of the string
@@ -27,27 +27,25 @@ SLEPC_EXTERN PetscErrorCode PCCreate_HPDDM(PC);
     Output Parameter:
 .   version - version string
 
-    Level: developer
+    Level: intermediate
 
 .seealso: SlepcGetVersionNumber()
 @*/
 PetscErrorCode SlepcGetVersion(char version[],size_t len)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
 #if (SLEPC_VERSION_RELEASE == 1)
-  ierr = PetscSNPrintf(version,len,"SLEPc Release Version %d.%d.%d, %s",SLEPC_VERSION_MAJOR,SLEPC_VERSION_MINOR,SLEPC_VERSION_SUBMINOR,SLEPC_VERSION_DATE);CHKERRQ(ierr);
+  PetscCall(PetscSNPrintf(version,len,"SLEPc Release Version %d.%d.%d, %s",SLEPC_VERSION_MAJOR,SLEPC_VERSION_MINOR,SLEPC_VERSION_SUBMINOR,SLEPC_VERSION_DATE));
 #else
-  ierr = PetscSNPrintf(version,len,"SLEPc Development GIT revision: %s  GIT Date: %s",SLEPC_VERSION_GIT,SLEPC_VERSION_DATE_GIT);CHKERRQ(ierr);
+  PetscCall(PetscSNPrintf(version,len,"SLEPc Development GIT revision: %s  GIT Date: %s",SLEPC_VERSION_GIT,SLEPC_VERSION_DATE_GIT));
 #endif
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@C
     SlepcGetVersionNumber - Gets the SLEPc version information from the library.
 
-    Not collective
+    Not Collective
 
     Output Parameters:
 +   major    - the major version
@@ -64,7 +62,7 @@ PetscErrorCode SlepcGetVersion(char version[],size_t len)
 
     This function can be called before SlepcInitialize().
 
-    Level: developer
+    Level: intermediate
 
 .seealso: SlepcGetVersion(), SlepcInitialize()
 @*/
@@ -74,7 +72,7 @@ PetscErrorCode SlepcGetVersionNumber(PetscInt *major,PetscInt *minor,PetscInt *s
   if (minor)    *minor    = SLEPC_VERSION_MINOR;
   if (subminor) *subminor = SLEPC_VERSION_SUBMINOR;
   if (release)  *release  = SLEPC_VERSION_RELEASE;
-  return 0;
+  return PETSC_SUCCESS;
 }
 
 /*
@@ -84,16 +82,15 @@ PetscErrorCode SlepcGetVersionNumber(PetscInt *major,PetscInt *minor,PetscInt *s
 */
 static PetscErrorCode SlepcPrintVersion(MPI_Comm comm)
 {
-  PetscErrorCode ierr;
   char           version[256];
 
   PetscFunctionBegin;
-  ierr = SlepcGetVersion(version,256);CHKERRQ(ierr);
-  ierr = (*PetscHelpPrintf)(comm,"%s\n",version);CHKERRQ(ierr);
-  ierr = (*PetscHelpPrintf)(comm,SLEPC_AUTHOR_INFO);CHKERRQ(ierr);
-  ierr = (*PetscHelpPrintf)(comm,"See docs/manual.html for help.\n");CHKERRQ(ierr);
-  ierr = (*PetscHelpPrintf)(comm,"SLEPc libraries linked from %s\n",SLEPC_LIB_DIR);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
+  PetscCall(SlepcGetVersion(version,256));
+  PetscCall((*PetscHelpPrintf)(comm,"%s\n",version));
+  PetscCall((*PetscHelpPrintf)(comm,SLEPC_AUTHOR_INFO));
+  PetscCall((*PetscHelpPrintf)(comm,"See docs/manual.html for help.\n"));
+  PetscCall((*PetscHelpPrintf)(comm,"SLEPc libraries linked from %s\n",SLEPC_LIB_DIR));
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*
@@ -103,13 +100,11 @@ static PetscErrorCode SlepcPrintVersion(MPI_Comm comm)
 */
 static PetscErrorCode SlepcPrintHelpIntro(MPI_Comm comm)
 {
-  PetscErrorCode  ierr;
-
   PetscFunctionBegin;
-  ierr = (*PetscHelpPrintf)(comm,"SLEPc help information includes that for the PETSc libraries, which provide\n");CHKERRQ(ierr);
-  ierr = (*PetscHelpPrintf)(comm,"low-level system infrastructure and linear algebra tools.\n");CHKERRQ(ierr);
-  ierr = (*PetscHelpPrintf)(comm,"----------------------------------------\n");CHKERRQ(ierr);
-  PetscFunctionReturn(0);
+  PetscCall((*PetscHelpPrintf)(comm,"SLEPc help information includes that for the PETSc libraries, which provide\n"));
+  PetscCall((*PetscHelpPrintf)(comm,"low-level system infrastructure and linear algebra tools.\n"));
+  PetscCall((*PetscHelpPrintf)(comm,"----------------------------------------\n"));
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /* ------------------------Nasty global variables -------------------------------*/
@@ -117,28 +112,26 @@ static PetscErrorCode SlepcPrintHelpIntro(MPI_Comm comm)
    Indicates whether SLEPc started PETSc, or whether it was
    already started before SLEPc was initialized.
 */
-PetscBool SlepcBeganPetsc = PETSC_FALSE;
+PetscBool SlepcBeganPetsc       = PETSC_FALSE;
 PetscBool SlepcInitializeCalled = PETSC_FALSE;
+PetscBool SlepcFinalizeCalled   = PETSC_FALSE;
 
 #if defined(PETSC_HAVE_DYNAMIC_LIBRARIES) && defined(PETSC_USE_SHARED_LIBRARIES)
 static PetscErrorCode SlepcLoadDynamicLibrary(const char *name,PetscBool *found)
 {
   char           libs[PETSC_MAX_PATH_LEN],dlib[PETSC_MAX_PATH_LEN];
-  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscStrcpy(libs,SLEPC_LIB_DIR);CHKERRQ(ierr);
-  ierr = PetscStrcat(libs,"/libslepc");CHKERRQ(ierr);
-  ierr = PetscStrcat(libs,name);CHKERRQ(ierr);
-  ierr = PetscDLLibraryRetrieve(PETSC_COMM_WORLD,libs,dlib,1024,found);CHKERRQ(ierr);
-  if (*found) {
-    ierr = PetscDLLibraryAppend(PETSC_COMM_WORLD,&PetscDLLibrariesLoaded,dlib);CHKERRQ(ierr);
-  }
-  PetscFunctionReturn(0);
+  PetscCall(PetscStrcpy(libs,SLEPC_LIB_DIR));
+  PetscCall(PetscStrlcat(libs,"/libslepc",sizeof(libs)));
+  PetscCall(PetscStrlcat(libs,name,sizeof(libs)));
+  PetscCall(PetscDLLibraryRetrieve(PETSC_COMM_WORLD,libs,dlib,sizeof(dlib),found));
+  if (*found) PetscCall(PetscDLLibraryAppend(PETSC_COMM_WORLD,&PetscDLLibrariesLoaded,dlib));
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 #endif
 
-#if defined(PETSC_HAVE_THREADSAFETY)
+#if defined(PETSC_USE_SINGLE_LIBRARY) && !(defined(PETSC_HAVE_DYNAMIC_LIBRARIES) && defined(PETSC_USE_SHARED_LIBRARIES))
 SLEPC_EXTERN PetscErrorCode STInitializePackage(void);
 SLEPC_EXTERN PetscErrorCode DSInitializePackage(void);
 SLEPC_EXTERN PetscErrorCode FNInitializePackage(void);
@@ -158,67 +151,67 @@ SLEPC_EXTERN PetscErrorCode LMEInitializePackage(void);
 */
 PetscErrorCode SlepcInitialize_DynamicLibraries(void)
 {
-#if (defined(PETSC_HAVE_DYNAMIC_LIBRARIES) && defined(PETSC_USE_SHARED_LIBRARIES)) || defined(PETSC_HAVE_THREADSAFETY)
-  PetscErrorCode ierr;
-#endif
-#if defined(PETSC_HAVE_DYNAMIC_LIBRARIES) && defined(PETSC_USE_SHARED_LIBRARIES)
-  PetscBool      found,preload;
-#endif
+  PetscBool      preload = PETSC_FALSE;
 
   PetscFunctionBegin;
-#if defined(PETSC_HAVE_DYNAMIC_LIBRARIES) && defined(PETSC_USE_SHARED_LIBRARIES)
-  preload = PETSC_FALSE;
-  ierr = PetscOptionsGetBool(NULL,NULL,"-dynamic_library_preload",&preload,NULL);CHKERRQ(ierr);
-  if (preload) {
-#if defined(PETSC_USE_SINGLE_LIBRARY)
-    ierr = SlepcLoadDynamicLibrary("",&found);CHKERRQ(ierr);
-    if (!found) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc dynamic library\nYou cannot move the dynamic libraries!");
-#else
-    ierr = SlepcLoadDynamicLibrary("sys",&found);CHKERRQ(ierr);
-    if (!found) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc sys dynamic library\nYou cannot move the dynamic libraries!");
-    ierr = SlepcLoadDynamicLibrary("eps",&found);CHKERRQ(ierr);
-    if (!found) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc EPS dynamic library\nYou cannot move the dynamic libraries!");
-    ierr = SlepcLoadDynamicLibrary("pep",&found);CHKERRQ(ierr);
-    if (!found) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc PEP dynamic library\nYou cannot move the dynamic libraries!");
-    ierr = SlepcLoadDynamicLibrary("nep",&found);CHKERRQ(ierr);
-    if (!found) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc NEP dynamic library\nYou cannot move the dynamic libraries!");
-    ierr = SlepcLoadDynamicLibrary("svd",&found);CHKERRQ(ierr);
-    if (!found) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc SVD dynamic library\nYou cannot move the dynamic libraries!");
-    ierr = SlepcLoadDynamicLibrary("mfn",&found);CHKERRQ(ierr);
-    if (!found) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc MFN dynamic library\nYou cannot move the dynamic libraries!");
-    ierr = SlepcLoadDynamicLibrary("lme",&found);CHKERRQ(ierr);
-    if (!found) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc LME dynamic library\nYou cannot move the dynamic libraries!");
-#endif
-  }
+#if defined(PETSC_HAVE_THREADSAFETY)
+  /* These must be all initialized here because it is not safe for individual threads to call these initialize routines */
+  preload = PETSC_TRUE;
 #endif
 
-#if defined(PETSC_HAVE_THREADSAFETY)
-  ierr = STInitializePackage();CHKERRQ(ierr);
-  ierr = DSInitializePackage();CHKERRQ(ierr);
-  ierr = FNInitializePackage();CHKERRQ(ierr);
-  ierr = BVInitializePackage();CHKERRQ(ierr);
-  ierr = RGInitializePackage();CHKERRQ(ierr);
-  ierr = EPSInitializePackage();CHKERRQ(ierr);
-  ierr = SVDInitializePackage();CHKERRQ(ierr);
-  ierr = PEPInitializePackage();CHKERRQ(ierr);
-  ierr = NEPInitializePackage();CHKERRQ(ierr);
-  ierr = MFNInitializePackage();CHKERRQ(ierr);
-  ierr = LMEInitializePackage();CHKERRQ(ierr);
+  PetscCall(PetscOptionsGetBool(NULL,NULL,"-library_preload",&preload,NULL));
+  if (preload) {
+#if defined(PETSC_HAVE_DYNAMIC_LIBRARIES) && defined(PETSC_USE_SHARED_LIBRARIES)
+    PetscBool found;
+#if defined(PETSC_USE_SINGLE_LIBRARY)
+    PetscCall(SlepcLoadDynamicLibrary("",&found));
+    PetscCheck(found,PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc dynamic library\nYou cannot move the dynamic libraries!");
+#else
+    PetscCall(SlepcLoadDynamicLibrary("sys",&found));
+    PetscCheck(found,PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc sys dynamic library\nYou cannot move the dynamic libraries!");
+    PetscCall(SlepcLoadDynamicLibrary("eps",&found));
+    PetscCheck(found,PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc EPS dynamic library\nYou cannot move the dynamic libraries!");
+    PetscCall(SlepcLoadDynamicLibrary("pep",&found));
+    PetscCheck(found,PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc PEP dynamic library\nYou cannot move the dynamic libraries!");
+    PetscCall(SlepcLoadDynamicLibrary("nep",&found));
+    PetscCheck(found,PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc NEP dynamic library\nYou cannot move the dynamic libraries!");
+    PetscCall(SlepcLoadDynamicLibrary("svd",&found));
+    PetscCheck(found,PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc SVD dynamic library\nYou cannot move the dynamic libraries!");
+    PetscCall(SlepcLoadDynamicLibrary("mfn",&found));
+    PetscCheck(found,PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc MFN dynamic library\nYou cannot move the dynamic libraries!");
+    PetscCall(SlepcLoadDynamicLibrary("lme",&found));
+    PetscCheck(found,PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc LME dynamic library\nYou cannot move the dynamic libraries!");
 #endif
+#else /* defined(PETSC_HAVE_DYNAMIC_LIBRARIES) && defined(PETSC_USE_SHARED_LIBRARIES) */
+#if defined(PETSC_USE_SINGLE_LIBRARY)
+  PetscCall(STInitializePackage());
+  PetscCall(DSInitializePackage());
+  PetscCall(FNInitializePackage());
+  PetscCall(BVInitializePackage());
+  PetscCall(RGInitializePackage());
+  PetscCall(EPSInitializePackage());
+  PetscCall(SVDInitializePackage());
+  PetscCall(PEPInitializePackage());
+  PetscCall(NEPInitializePackage());
+  PetscCall(MFNInitializePackage());
+  PetscCall(LMEInitializePackage());
+#else
+  SETERRQ(PETSC_COMM_WORLD,PETSC_ERR_SUP,"Cannot use -library_preload with multiple static SLEPc libraries");
+#endif
+#endif /* defined(PETSC_HAVE_DYNAMIC_LIBRARIES) && defined(PETSC_USE_SHARED_LIBRARIES) */
+  }
 
 #if defined(SLEPC_HAVE_HPDDM)
-  ierr = KSPRegister(KSPHPDDM,KSPCreate_HPDDM);CHKERRQ(ierr);
-  ierr = PCRegister(PCHPDDM,PCCreate_HPDDM);CHKERRQ(ierr);
+  PetscCall(KSPRegister(KSPHPDDM,KSPCreate_HPDDM));
+  PetscCall(PCRegister(PCHPDDM,PCCreate_HPDDM));
 #endif
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode SlepcCitationsInitialize()
+PetscErrorCode SlepcCitationsInitialize(void)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
-  ierr = PetscCitationsRegister("@Article{slepc-toms,\n"
+  PetscCall(PetscCitationsRegister("@Article{slepc-toms,\n"
     "   author = \"Vicente Hernandez and Jose E. Roman and Vicente Vidal\",\n"
     "   title = \"{SLEPc}: A Scalable and Flexible Toolkit for the Solution of Eigenvalue Problems\",\n"
     "   journal = \"{ACM} Trans. Math. Software\",\n"
@@ -227,15 +220,15 @@ PetscErrorCode SlepcCitationsInitialize()
     "   pages = \"351--362\",\n"
     "   year = \"2005,\"\n"
     "   doi = \"https://doi.org/10.1145/1089014.1089019\"\n"
-    "}\n",NULL);CHKERRQ(ierr);
-  ierr = PetscCitationsRegister("@TechReport{slepc-manual,\n"
-    "   author = \"J. E. Roman and C. Campos and E. Romero and A. Tomas\",\n"
+    "}\n",NULL));
+  PetscCall(PetscCitationsRegister("@TechReport{slepc-manual,\n"
+    "   author = \"J. E. Roman and C. Campos and L. Dalcin and E. Romero and A. Tomas\",\n"
     "   title = \"{SLEPc} Users Manual\",\n"
-    "   number = \"DSIC-II/24/02 - Revision 3.13\",\n"
+    "   number = \"DSIC-II/24/02 - Revision 3.19\",\n"
     "   institution = \"D. Sistemes Inform\\`atics i Computaci\\'o, Universitat Polit\\`ecnica de Val\\`encia\",\n"
-    "   year = \"2020\"\n"
-    "}\n",NULL);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
+    "   year = \"2023\"\n"
+    "}\n",NULL));
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@C
@@ -252,7 +245,7 @@ PetscErrorCode SlepcCitationsInitialize()
           (use NULL for default)
 -  help - [optional] Help message to print, use NULL for no message
 
-   Fortran Note:
+   Fortran Notes:
    Fortran syntax is very similar to that of PetscInitialize()
 
    Level: beginner
@@ -261,31 +254,26 @@ PetscErrorCode SlepcCitationsInitialize()
 @*/
 PetscErrorCode SlepcInitialize(int *argc,char ***args,const char file[],const char help[])
 {
-  PetscErrorCode ierr;
   PetscBool      flg;
 
   PetscFunctionBegin;
-  if (SlepcInitializeCalled) PetscFunctionReturn(0);
-  ierr = PetscSetHelpVersionFunctions(SlepcPrintHelpIntro,SlepcPrintVersion);CHKERRQ(ierr);
-  ierr = PetscInitialized(&flg);CHKERRQ(ierr);
+  if (SlepcInitializeCalled) PetscFunctionReturn(PETSC_SUCCESS);
+  PetscCall(PetscSetHelpVersionFunctions(SlepcPrintHelpIntro,SlepcPrintVersion));
+  PetscCall(PetscInitialized(&flg));
   if (!flg) {
-    ierr = PetscInitialize(argc,args,file,help);CHKERRQ(ierr);
+    PetscCall(PetscInitialize(argc,args,file,help));
     SlepcBeganPetsc = PETSC_TRUE;
   }
 
-  ierr = SlepcCitationsInitialize();CHKERRQ(ierr);
+  PetscCall(SlepcCitationsInitialize());
 
   /* Load the dynamic libraries (on machines that support them), this registers all the solvers etc. */
-  ierr = SlepcInitialize_DynamicLibraries();CHKERRQ(ierr);
-
-#if defined(PETSC_HAVE_DRAND48)
-  /* work-around for Cygwin drand48() initialization bug */
-  srand48(0);
-#endif
+  PetscCall(SlepcInitialize_DynamicLibraries());
 
   SlepcInitializeCalled = PETSC_TRUE;
-  ierr = PetscInfo(0,"SLEPc successfully started\n");CHKERRQ(ierr);
-  PetscFunctionReturn(0);
+  SlepcFinalizeCalled   = PETSC_FALSE;
+  PetscCall(PetscInfo(0,"SLEPc successfully started\n"));
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@C
@@ -300,15 +288,20 @@ PetscErrorCode SlepcInitialize(int *argc,char ***args,const char file[],const ch
 @*/
 PetscErrorCode SlepcFinalize(void)
 {
-  PetscErrorCode ierr = 0;
-
   PetscFunctionBegin;
-  ierr = PetscInfo(0,"SlepcFinalize() called\n");CHKERRQ(ierr);
+  if (PetscUnlikely(!SlepcInitializeCalled)) {
+    fprintf(PETSC_STDOUT,"SlepcInitialize() must be called before SlepcFinalize()\n");
+    PetscStackClearTop;
+    return PETSC_ERR_ARG_WRONGSTATE;
+  }
+  PetscCall(PetscInfo(NULL,"SlepcFinalize() called\n"));
   if (SlepcBeganPetsc) {
-    ierr = PetscFinalize();
+    PetscCall(PetscFinalize());
+    SlepcBeganPetsc = PETSC_FALSE;
   }
   SlepcInitializeCalled = PETSC_FALSE;
-  PetscFunctionReturn(ierr);
+  SlepcFinalizeCalled   = PETSC_TRUE;
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@C
@@ -323,18 +316,20 @@ PetscErrorCode SlepcFinalize(void)
 @*/
 PetscErrorCode SlepcInitializeNoArguments(void)
 {
-  PetscErrorCode ierr;
-  int            argc = 0;
-  char           **args = 0;
+  int  argc = 0;
+  char **args = NULL;
 
   PetscFunctionBegin;
-  ierr = SlepcInitialize(&argc,&args,NULL,NULL);
-  PetscFunctionReturn(ierr);
+  PetscCall(SlepcInitialize(&argc,&args,NULL,NULL));
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    SlepcInitialized - Determine whether SLEPc is initialized.
 
+   Output Parameter:
+.  isInitialized - the result
+
    Level: beginner
 
 .seealso: SlepcInitialize(), SlepcInitializeFortran()
@@ -342,33 +337,54 @@ PetscErrorCode SlepcInitializeNoArguments(void)
 PetscErrorCode SlepcInitialized(PetscBool *isInitialized)
 {
   PetscFunctionBegin;
-  PetscValidBoolPointer(isInitialized,1);
   *isInitialized = SlepcInitializeCalled;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
+}
+
+/*@
+   SlepcFinalized - Determine whether SlepcFinalize() has been called.
+
+   Output Parameter:
+.  isFinalized - the result
+
+   Level: developer
+
+.seealso: SlepcFinalize()
+@*/
+PetscErrorCode SlepcFinalized(PetscBool *isFinalized)
+{
+  PetscFunctionBegin;
+  *isFinalized = SlepcFinalizeCalled;
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 PETSC_EXTERN PetscBool PetscBeganMPI;
 
-/*
+/*@C
    SlepcInitializeNoPointers - Calls SlepcInitialize() from C/C++ without the pointers
    to argc and args (analogue to PetscInitializeNoPointers).
 
    Collective
 
+   Input Parameters:
++  argc - count of number of command line arguments
+.  args - the command line arguments
+.  file - [optional] PETSc database file, defaults to ~username/.petscrc
+          (use NULL for default)
+-  help - [optional] Help message to print, use NULL for no message
+
    Level: advanced
 
 .seealso: SlepcInitialize()
-*/
-PetscErrorCode SlepcInitializeNoPointers(int argc,char **args,const char *filename,const char *help)
+@*/
+PetscErrorCode SlepcInitializeNoPointers(int argc,char **args,const char *file,const char *help)
 {
-  PetscErrorCode ierr;
   int            myargc = argc;
   char           **myargs = args;
 
   PetscFunctionBegin;
-  ierr = SlepcInitialize(&myargc,&myargs,filename,help);CHKERRQ(ierr);
-  ierr = PetscPopSignalHandler();CHKERRQ(ierr);
+  PetscCall(SlepcInitialize(&myargc,&myargs,file,help));
+  PetscCall(PetscPopSignalHandler());
   PetscBeganMPI = PETSC_FALSE;
-  PetscFunctionReturn(ierr);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
-
